#!/bin/bash -e

# This script is used by the gitlab CI of this repository to run a 
# test of adcern lib functionalities
# Requirements:
#     - spark_etl libs (distributed in docker image)
#     - Kerberos authentication (passed via CI)
#     - cvmfs spark libs (exposed via cvmfs)

# In order to access the needed libraries from cvmfs, 
# a cvmfs service is started with docker-compose 
# 
# In order to run the same script manually, assuming only docker available, run
#
# CI_PROJECT_DIR=`pwd | sed -e 's@/tests/.*@@'`
# docker run --rm -e CI_USER=$CI_USER -e CI_USER_PASSWD=$CI_USER_PASSWD -e CI_PROJECT_DIR=${CI_PROJECT_DIR} 
#         -v /tmp:/tmp -v /builds:/builds -v `pwd`:/work -v /var/run/docker.sock:/var/run/docker.sock 
#          gitlab-registry.cern.ch/cloud-infrastructure/data-analytics/compose:v1.0 bash -c '. /work/ci_test_script.sh; start_docker_compose'
#
# Consider to open the Spark connection ports in iptables
#
# sudo iptables -I INPUT -p tcp -m multiport --dports 5001:6000 -m comment --comment "00200 firewall for hadoop jobs" -j ACCEPT
# sudo iptables -I DOCKER-USER -p tcp -m multiport --dports 5001:6000 -m comment --comment "00200 firewall for hadoop jobs" -j ACCEPT
#

function start_docker_compose() {
    rm -f compose.log cnt_adcern.log myfluent.log

    echo -e "\n------- Creating Kerberos Directory and token ----\n"
    mkdir -p $KRB5DIR
    kdestroy -c $KRB5CCNAME ; echo $CI_USER_PASSWD | kinit -c $KRB5CCNAME $CI_USER@CERN.CH; klist -c $KRB5CCNAME
    ls -l $KRB5DIR
    ls /eos/project-i/it-cloud-data-analytics/

    echo -e "\n------- DUMP docker-compose.yml ----\n"
    docker-compose -f docker-compose.yml config
    echo -e "\n------- END docker-compose.yml ----\n"
    docker-compose -p adcern down --remove-orphans --volumes

    docker-compose pull
    echo -e "\n------- START docker-compose.yml ----\n"
    docker-compose -f docker-compose.yml -p adcern up --remove-orphans --renew-anon-volumes -d
    echo -e "\n------- Saving the logs in compose.log and cnt_adcern.log----\n"
    docker-compose -p adcern logs -f 2>&1 >> compose.log &
    docker logs cnt_adcern -f 2>&1 >> cnt_adcern.log &
    echo -e "\n------- Running Dockers ----\n"
    docker ps
}

function comparison(){
    echo -e "\n------- The function comparison started! Let's wait for 2 minutes. ----\n"
    sleep 2m
    echo -e "\n------- The function comparison keeps waiting also for cnt_adcern to finish. ----\n"

    while [ "$( docker container inspect -f '{{.State.Status}}' cnt_adcern )" == "running" ]; do
        echo -e "\n------------------------------------------------------------------------------\n"
        echo -e "\n Container cnt_adcern still running....last 5 lines of logs:\n"
        docker logs cnt_adcern --tail=5
        echo -e "\n------------------------------------------------------------------------------\n"
        echo -e "\n Sleeping for 1 minute....\n"
        sleep 1m        
    done
    echo -e "\n------- The function comparison has finished waiting. ----\n"
    docker run --rm --user=root --net=adcern_default --name result_reader -v $(pwd):/local_folder ${CI_REGISTRY_IMAGE}/sparknotebook:qa python3 /local_folder/result_comparison.py read_result
}

function stop_docker_compose(){
    docker-compose -p adcern down --remove-orphans --volumes  # First remove containers
    docker container prune -f
    docker-compose -p adcern down --rmi all --remove-orphans --volumes # Then remove images
    cat compose.log
    cat cnt_adcern.log
}

export WORK_DIR=$(readlink -f $(dirname $BASH_SOURCE))
echo WORK_DIR $WORK_DIR

cd $WORK_DIR
export CI_JOB_ID=${CI_JOB_ID:-noCI}
export CI_COMMIT_BRANCH=${CI_COMMIT_BRANCH:-noCI}
export COMMIT_TAG=${CI_COMMIT_TAG:-$CI_COMMIT_BRANCH}
export CVMFSDIR=/builds/${COMMIT_TAG}/cvmfs-${CI_JOB_ID}
export KRB5DIR=/builds/${COMMIT_TAG}/krb5-${CI_JOB_ID}
export IMAGE_TAG=${COMMIT_TAG}
export KRB5CCNAME=$KRB5DIR/krb5cc_docker
